# encoding: utf-8

require 'spec_helper'

describe AppStack do
  describe '#load_config' do
    subject { AppStack::App.new(examples('sample_config/app_stack.yml')) }
    it 'parses any yaml file' do
      subject.load_config
      subject.config.should be_instance_of(AppStack::Configuration)
      subject.config.export.should be_instance_of(Array)
    end
  end
end

describe AppStack::Configuration do
  describe '#initialize' do
    it 'raise error for non-exists keys' do
      expect do
        AppStack::Configuration.new(examples('sample_config/wrong_key.yml'))
      end.to raise_error(AppStack::ParseError)
    end

    it 'raise error for keys with wrong data type' do
      expect do
        AppStack::Configuration.new(examples('sample_config/wrong_type.yml'))
      end.to raise_error(AppStack::ParseError)
    end

    it 'return default config for directory without config file' do
      conf = AppStack::Configuration.new
      conf.export.should be_instance_of(Array)
      conf.export.size.should be(0)
    end
  end

  describe '#parse_export_files' do
    subject(:app) do
      app = AppStack::App.new(examples('sample_config/app_stack.yml'))
      app.validate_config!
      app
    end

    it 'raise error for wrong export definition' do
      expect do
        AppStack::App.new(examples('sample_config/hash_export.yml'))
          .validate_config!
      end.to raise_error(AppStack::ParseError)
    end

    its(:export_groups) { should be_instance_of(Hash) }

    it 'catch default export group files' do
      app.export_groups['default']
        .should include('app/lib/bases/web_application.rb')
      app.export_groups['default']
        .should include('features/step_definitions/web_steps.rb')
      app.export_groups['default']
        .should include('public/assets/definition.yml')

    end

    it 'catch explicit named export group files' do
      app.export_groups['gemfile']
        .should include('Gemfile')
      app.export_groups['cucumber'].should include('lib/task/cucumber.rake')
      app.export_groups['cucumber'] .should include('features/support/*')
    end
  end

  describe '#sync_list' do
    subject(:app) do
      app = AppStack::App.new(examples('sample_config/app_stack.yml'))
      app.validate_config!
      app.sync_list
    end

    it 'include default group for string setting' do
      subject['la-mongoid'].should include('default')
    end

    it 'include groups for array settings' do
      subject['la-assets-helper'].should include('default')
    end

    it 'include single file hash settings' do
      subject['la-assets-helper'][1]['Rakefile'].should eq('Rakefile')
    end

    it 'include multi group settings' do
      subject['la-sinatra'].size.should eq(2)
      subject['la-sinatra'].should include('tests')
    end
  end
end
